require 'bunny'
require 'json'
require 'teamforge'
require 'rest-client'

# Class to post messeges to orchestrate
class OrchestrateClient
  private_class_method :new # force callers to use connect

  attr_accessor :connection, :queue, :exchange, :connection_string

  def initialize(server, user, pass)
    # accept abc.com, amqp://abc.com, amqp://abc.com:5678, or abc.com:5678
    matched = server.match %r{\A(amqp://)?([^:]*)(:(\d+))?\z}
    proto, host, _, port = matched[1, 4]
    proto ||= 'amqp://'
    port ||= 5672
    self.connection_string = "#{proto}#{user}:#{pass}@#{host}:#{port}"
  end 

  def self.connect(server, user, pass, queue_name)
    obj = new(server, user, pass)
    obj.connection = Bunny.new(obj.connection_string, connection_timeout: 10) 
    obj.connection.start
    channel = obj.connection.create_channel
    obj.queue = channel.queue(queue_name, durable: true)
    obj.exchange = channel.default_exchange

    return obj unless block_given?

    begin
     yield obj 
    ensure
     obj.close
    end 
  end 

  def post(body)
    exchange.publish(body, routing_key: queue.name)
  end 

  def close
    connection.close
  end 
end

module CustomHandler
  class AllDetails < Chef::Handler
    def initialize(config = {})
      @config = config
      @config
    end

    def check_schema
      TeamForge.server = @config[:ctf_url]
      TeamForge.use_ssl = @config[:ctf_url].start_with? 'https://'
      ctf_soap_session = TeamForge::Login.new(@config[:ctf_user], @config[:ctf_pass]).request
      ip = @config[:orc_url] ? @config[:orc_url] : "#{@config[:ctf_url]}/orc"
      ip_ep = "#{ip}/api/#{@config[:api_version]}/schemas"
      req_url = "#{ip_ep}/#{@config[:schema_id]}/#{@config[:schema_version]}"
      begin
        RestClient::Resource.new(req_url, headers: {content_type: :json, accept: :json, 'X-Orchestrate-Session' => ctf_soap_session}).get
      rescue RestClient::ResourceNotFound
        RestClient::Resource.new(ip_ep, headers: {content_type: :json, accept: :json, 'X-Orchestrate-Session' => ctf_soap_session}).post(schema_data)
      end
    end

    def schema_data
    <<-EOF
    {
      "api_version": "#{@config[:api_version]}",
      "custom_schema": {
        "event_time": "#{Time.now.getgm.iso8601}",
        "name": "#{@config[:schema_name]}",
        "schema_id": "#{@config[:schema_id]}",
        "schema_version": #{@config[:schema_version]},
        "fields": [
          "event_time",
          "schema_id",
          "remote_id",
          "schema_version",
          "created_by",
          "summary",
          "description",
          "associated_remote_id",
          "status_name",
          "status_type",
          "event_type",
          "node_name",
          "node_local_time"
        ],
        "important": [
          "node_name",
          "node_local_time",
          "summary"
        ],
        "parsed": [
          "description"
        ],
        "required": [
          "remote_id",
          "event_time",
          "schema_id",
          "schema_version",
          "associated_remote_id",
          "status_type",
          "summary"
        ],
        "hidden": []
      }
    }
    EOF
    end

    def orc_messages
    [{
      'api_version' => @config[:api_version].to_s,
      'source_association_key' => @config[:src_assoc_key],
      'custom_data' => {
        'schema_id' => @config[:schema_id],
        'schema_version' => @config[:schema_version],
        'event_time' => run_status.start_time.getgm.iso8601,
        'node_local_time' => run_status.start_time,
        'created_by' => 'Chef',
        'event_type' => 'artifact_deployment',
        'status_type' => 'open',
        'status_name' => 'Deployment began successfully',
        'remote_id' => "#{@config[:node_name]}-#{run_status.start_time.getgm.iso8601}",
        'associated_remote_id' => @associated_remote_id,
        'node_name' => @config[:node_name],
        'summary' => 'Deployment in progress',
        'description' => "Deployment on #{@config[:node_name]} began at #{run_status.start_time}."
      }
     },
     {
      'api_version' => @config[:api_version].to_s,
      'source_association_key' => @config[:src_assoc_key],
      'custom_data' => {
        'schema_id' => @config[:schema_id],
        'schema_version' => @config[:schema_version],
        'event_time' => run_status.end_time.getgm.iso8601,
        'node_local_time' => run_status.end_time,
        'created_by' => 'Chef',
        'event_type' => 'artifact_deployment',
        'status_type' => (run_status.success? ? 'success' : 'failed'),
        'status_name' => "Deployment #{run_status.success? ? 'succeded' : 'failed'}.",
        'remote_id' => "#{@config[:node_name]}-#{run_status.end_time.getgm.iso8601}",
        'associated_remote_id' => @associated_remote_id,
        'node_name' => @config[:node_name],
        'summary' => 'Deployment complete',
        'description' => "Deployment on #{@config[:node_name]} #{run_status.success? ? 'succeeded' : 'failed'} at #{run_status.end_time}."
      }
     }]
    end

    def report
      if @config[:src_url]
        tmp_associated_remote_id = RestClient::Resource.new(@config[:src_url], user: 'anonymous', password: 'anonymous').get do |rsp, _, rst|
          [301, 302, 307].include?(rsp.code) ? rst['location'] : @config[:src_url]
        end
        @associated_remote_id = tmp_associated_remote_id.split('/')[9..-1].join('/')
        post_to_orc = true
      elsif @config[:associated_remote_id]
        @associated_remote_id = @config[:associated_remote_id]
        post_to_orc = true
      else
        post_to_orc = false
      end

      if post_to_orc
        check_schema
        orc_messages.each do |orc_message|
          OrchestrateClient.connect(@config[:server], @config[:user], @config[:password], @config[:queue_name]){|orc| orc.post(orc_message.to_json)}
        end
      end
    end
  end
end
